@@ -108,7 +108,7 @@ static igraph_error_t igraph_i_connected_components_weak(
 ) {
 
     igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    char *already_added;
+    bool *already_added;
     igraph_integer_t first_node, act_cluster_size = 0, no_of_clusters = 0;
 
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
@@ -116,10 +116,8 @@ static igraph_error_t igraph_i_connected_components_weak(
     igraph_integer_t i;
     igraph_vector_int_t neis = IGRAPH_VECTOR_NULL;
 
-    already_added = IGRAPH_CALLOC(no_of_nodes, char);
-    if (already_added == 0) {
-        IGRAPH_ERROR("Cannot calculate weakly connected components.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
+    already_added = IGRAPH_CALLOC(no_of_nodes, bool);
+    IGRAPH_CHECK_OOM(already_added, "Insufficient memory for calculating weakly connected components.");
     IGRAPH_FINALLY(igraph_free, already_added);
 
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, no_of_nodes > 100000 ? 10000 : no_of_nodes / 10);
@@ -136,12 +134,12 @@ static igraph_error_t igraph_i_connected_components_weak(
     /* The algorithm */
 
     for (first_node = 0; first_node < no_of_nodes; ++first_node) {
-        if (already_added[first_node] == 1) {
+        if (already_added[first_node]) {
             continue;
         }
         IGRAPH_ALLOW_INTERRUPTION();
 
-        already_added[first_node] = 1;
+        already_added[first_node] = true;
         act_cluster_size = 1;
         if (membership) {
             VECTOR(*membership)[first_node] = no_of_clusters;
@@ -154,11 +152,11 @@ static igraph_error_t igraph_i_connected_components_weak(
             igraph_integer_t nei_count = igraph_vector_int_size(&neis);
             for (i = 0; i < nei_count; i++) {
                 igraph_integer_t neighbor = VECTOR(neis)[i];
-                if (already_added[neighbor] == 1) {
+                if (already_added[neighbor]) {
                     continue;
                 }
                 IGRAPH_CHECK(igraph_dqueue_int_push(&q, neighbor));
-                already_added[neighbor] = 1;
+                already_added[neighbor] = true;
                 act_cluster_size++;
                 if (membership) {
                     VECTOR(*membership)[neighbor] = no_of_clusters;
@@ -487,7 +485,7 @@ static igraph_error_t igraph_is_connected_weak(const igraph_t *graph, igraph_boo
 
         igraph_integer_t actnode = igraph_dqueue_int_pop(&q);
 
-        IGRAPH_CHECK(igraph_neighbors(graph, &neis, (igraph_integer_t) actnode, IGRAPH_ALL));
+        IGRAPH_CHECK(igraph_neighbors(graph, &neis, actnode, IGRAPH_ALL));
         igraph_integer_t nei_count = igraph_vector_int_size(&neis);
 
         for (igraph_integer_t i = 0; i < nei_count; i++) {
@@ -551,7 +549,7 @@ void igraph_decompose_destroy(igraph_vector_ptr_t *complist) {
     for (i = 0; i < n; i++) {
         if (VECTOR(*complist)[i] != 0) {
             igraph_destroy(VECTOR(*complist)[i]);
-            igraph_free(VECTOR(*complist)[i]);
+            IGRAPH_FREE(VECTOR(*complist)[i]);
         }
     }
 }
@@ -731,7 +729,6 @@ static igraph_error_t igraph_i_decompose_strong(const igraph_t *graph,
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
 
     igraph_integer_t no_of_clusters = 0;
-    igraph_integer_t act_cluster_size;
 
     igraph_vector_int_t out = IGRAPH_VECTOR_NULL;
     const igraph_vector_int_t* tmp;
@@ -853,7 +850,6 @@ static igraph_error_t igraph_i_decompose_strong(const igraph_t *graph,
 
         /* this node is gone for any future components */
         VECTOR(next_nei)[grandfather] = 1;
-        act_cluster_size = 1;
 
         /* add to component */
         IGRAPH_CHECK(igraph_vector_int_push_back(&verts, grandfather));
@@ -879,7 +875,6 @@ static igraph_error_t igraph_i_decompose_strong(const igraph_t *graph,
                 }
                 IGRAPH_CHECK(igraph_dqueue_int_push(&q, neighbor));
                 VECTOR(next_nei)[neighbor] = 1;
-                act_cluster_size++;
 
                 /* add to component */
                 IGRAPH_CHECK(igraph_vector_int_push_back(&verts, neighbor));
@@ -1029,30 +1024,23 @@ igraph_error_t igraph_biconnected_components(const igraph_t *graph,
     igraph_stack_int_t path;
     igraph_stack_int_t edgestack;
     igraph_inclist_t inclist;
-    igraph_integer_t i, counter, rootdfs = 0;
+    igraph_integer_t counter, rootdfs = 0;
     igraph_vector_int_t vertex_added;
     igraph_integer_t comps = 0;
     igraph_vector_int_list_t *mycomponents = components, vcomponents;
 
-    IGRAPH_CHECK(igraph_vector_int_init(&nextptr, no_of_nodes));
-    IGRAPH_FINALLY(igraph_vector_int_destroy, &nextptr);
-    IGRAPH_CHECK(igraph_vector_int_init(&num, no_of_nodes));
-    IGRAPH_FINALLY(igraph_vector_int_destroy, &num);
-    IGRAPH_CHECK(igraph_vector_int_init(&low, no_of_nodes));
-    IGRAPH_FINALLY(igraph_vector_int_destroy, &low);
-    IGRAPH_CHECK(igraph_vector_bool_init(&found, no_of_nodes));
-    IGRAPH_FINALLY(igraph_vector_bool_destroy, &found);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&nextptr, no_of_nodes);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&num, no_of_nodes);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&low, no_of_nodes);
+    IGRAPH_VECTOR_BOOL_INIT_FINALLY(&found, no_of_nodes);
 
-    IGRAPH_CHECK(igraph_stack_int_init(&path, 100));
-    IGRAPH_FINALLY(igraph_stack_int_destroy, &path);
-    IGRAPH_CHECK(igraph_stack_int_init(&edgestack, 100));
-    IGRAPH_FINALLY(igraph_stack_int_destroy, &edgestack);
+    IGRAPH_STACK_INT_INIT_FINALLY(&path, 100);
+    IGRAPH_STACK_INT_INIT_FINALLY(&edgestack, 100);
 
     IGRAPH_CHECK(igraph_inclist_init(graph, &inclist, IGRAPH_ALL, IGRAPH_LOOPS_TWICE));
     IGRAPH_FINALLY(igraph_inclist_destroy, &inclist);
 
-    IGRAPH_CHECK(igraph_vector_int_init(&vertex_added, no_of_nodes));
-    IGRAPH_FINALLY(igraph_vector_int_destroy, &vertex_added);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&vertex_added, no_of_nodes);
 
     if (no) {
         *no = 0;
@@ -1074,7 +1062,7 @@ igraph_error_t igraph_biconnected_components(const igraph_t *graph,
         IGRAPH_VECTOR_INT_LIST_INIT_FINALLY(mycomponents, 0);
     }
 
-    for (i = 0; i < no_of_nodes; i++) {
+    for (igraph_integer_t i = 0; i < no_of_nodes; i++) {
 
         if (VECTOR(low)[i] != 0) {
             continue;    /* already visited */
@@ -1134,7 +1122,7 @@ igraph_error_t igraph_biconnected_components(const igraph_t *graph,
                         /*------------------------------------*/
                         /* Record the biconnected component just found */
                         if (tree_edges || mycomponents) {
-                            igraph_vector_int_t *v = 0, *v2 = 0;
+                            igraph_vector_int_t *v, *v2;
                             comps++;
                             if (tree_edges) {
                                 IGRAPH_CHECK(igraph_vector_int_list_push_back_new(tree_edges, &v));
