@@ -120,7 +120,7 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
     igraph_dqueue_int_t Q;
     igraph_integer_t no_of_nodes = igraph_vcount(graph);
     igraph_integer_t actroot = 0;
-    igraph_vector_char_t added;
+    igraph_vector_bool_t added;
 
     igraph_lazy_adjlist_t adjlist;
 
@@ -131,38 +131,28 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
     igraph_integer_t noroots = roots ? igraph_vector_int_size(roots) : 1;
 
     if (!roots && (root < 0 || root >= no_of_nodes)) {
-        IGRAPH_ERROR("Invalid root vertex in BFS", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Invalid root vertex in BFS.", IGRAPH_EINVVID);
     }
 
-    if (roots && noroots > 0) {
-        igraph_integer_t min, max;
-        igraph_vector_int_minmax(roots, &min, &max);
-        if (min < 0 || max >= no_of_nodes) {
-            IGRAPH_ERROR("Invalid root vertex in BFS", IGRAPH_EINVAL);
-        }
+    if (roots && !igraph_vector_int_isininterval(roots, 0, no_of_nodes-1)) {
+        IGRAPH_ERROR("Invalid root vertex in BFS.", IGRAPH_EINVVID);
     }
 
-    if (restricted && igraph_vector_int_size(restricted) > 0) {
-        igraph_integer_t min, max;
-        igraph_vector_int_minmax(restricted, &min, &max);
-        if (min < 0 || max >= no_of_nodes) {
-            IGRAPH_ERROR("Invalid vertex ID in restricted set", IGRAPH_EINVAL);
-        }
+    if (restricted && !igraph_vector_int_isininterval(restricted, 0, no_of_nodes-1)) {
+        IGRAPH_ERROR("Invalid vertex ID in restricted set.", IGRAPH_EINVVID);
     }
 
     if (mode != IGRAPH_OUT && mode != IGRAPH_IN &&
         mode != IGRAPH_ALL) {
-        IGRAPH_ERROR("Invalid mode argument", IGRAPH_EINVMODE);
+        IGRAPH_ERROR("Invalid mode argument.", IGRAPH_EINVMODE);
     }
 
     if (!igraph_is_directed(graph)) {
         mode = IGRAPH_ALL;
     }
 
-    IGRAPH_CHECK(igraph_vector_char_init(&added, no_of_nodes));
-    IGRAPH_FINALLY(igraph_vector_char_destroy, &added);
-    IGRAPH_CHECK(igraph_dqueue_int_init(&Q, 100));
-    IGRAPH_FINALLY(igraph_dqueue_int_destroy, &Q);
+    IGRAPH_VECTOR_BOOL_INIT_FINALLY(&added, no_of_nodes);
+    IGRAPH_DQUEUE_INT_INIT_FINALLY(&Q, 100);
 
     IGRAPH_CHECK(igraph_lazy_adjlist_init(graph, &adjlist, mode, IGRAPH_LOOPS, IGRAPH_MULTIPLE));
     IGRAPH_FINALLY(igraph_lazy_adjlist_destroy, &adjlist);
@@ -172,10 +162,10 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
        the restricted set, but are to be used as 'root' vertices. */
     if (restricted) {
         igraph_integer_t i, n = igraph_vector_int_size(restricted);
-        igraph_vector_char_fill(&added, 1);
+        igraph_vector_bool_fill(&added, true);
         for (i = 0; i < n; i++) {
             igraph_integer_t v = VECTOR(*restricted)[i];
-            VECTOR(added)[v] = 0;
+            VECTOR(added)[v] = false;
         }
     }
 
@@ -225,7 +215,7 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
         }
         IGRAPH_CHECK(igraph_dqueue_int_push(&Q, actroot));
         IGRAPH_CHECK(igraph_dqueue_int_push(&Q, 0));
-        VECTOR(added)[actroot] = 1;
+        VECTOR(added)[actroot] = true;
         if (parents) {
             VECTOR(*parents)[actroot] = -1;
         }
@@ -258,7 +248,7 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
             for (i = 0; i < n; i++) {
                 igraph_integer_t nei = VECTOR(*neis)[i];
                 if (! VECTOR(added)[nei]) {
-                    VECTOR(added)[nei] = 1;
+                    VECTOR(added)[nei] = true;
                     IGRAPH_CHECK(igraph_dqueue_int_push(&Q, nei));
                     IGRAPH_CHECK(igraph_dqueue_int_push(&Q, actdist + 1));
                     if (parents) {
@@ -293,7 +283,7 @@ cleanup:
 
     igraph_lazy_adjlist_destroy(&adjlist);
     igraph_dqueue_int_destroy(&Q);
-    igraph_vector_char_destroy(&added);
+    igraph_vector_bool_destroy(&added);
     IGRAPH_FINALLY_CLEAN(3);
 
     return IGRAPH_SUCCESS;
