@@ -117,23 +117,21 @@
 
 */
 
-#include <stdio.h>
-#include <string.h>
-
 #include "igraph_types.h"
 #include "igraph_memory.h"
 #include "igraph_error.h"
-#include "config.h"
 
-#include "core/math.h"
 #include "io/lgl-header.h"
 #include "io/parsers/lgl-parser.h"
 #include "io/parsers/lgl-lexer.h"
+#include "io/parse_utils.h"
 #include "internal/hacks.h"
 
+#include <stdio.h>
+#include <string.h>
+
 int igraph_lgl_yyerror(YYLTYPE* locp, igraph_i_lgl_parsedata_t *context,
                        const char *s);
-igraph_real_t igraph_lgl_get_number(const char *str, long int len);
 
 #define scanner context->scanner
 
@@ -160,8 +158,8 @@ igraph_real_t igraph_lgl_get_number(const char *str, long int len);
 typedef union YYSTYPE
 
 {
-  long int edgenum;
-  double weightnum;
+  igraph_integer_t edgenum;
+  igraph_real_t weightnum;
 }
 /* Line 193 of yacc.c.  */
 
@@ -477,8 +475,8 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    94,    94,    95,    96,    99,   101,   103,   103,   105,
-     110,   119,   124
+       0,    92,    92,    93,    94,    97,    99,   101,   101,   103,
+     108,   117,   127
 };
 #endif
 
@@ -487,9 +485,9 @@ static const yytype_uint8 yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
 {
-  "\"end of file\"", "error", "$undefined", "ALNUM", "NEWLINE", "\"#\"",
-  "ERROR", "$accept", "input", "vertex", "vertexdef", "edges", "edge",
-  "edgeid", "weight", 0
+  "\"end of file\"", "error", "$undefined", "\"alphanumeric\"",
+  "\"end of line\"", "\"#\"", "ERROR", "$accept", "input", "vertex",
+  "vertexdef", "edges", "edge", "edgeid", "weight", 0
 };
 #endif
 
@@ -1418,34 +1416,44 @@ yyreduce:
   case 9:
 
     {
-             igraph_vector_push_back(context->vector, context->actvertex);
-             igraph_vector_push_back(context->vector, (yyvsp[(1) - (2)].edgenum));
-             igraph_vector_push_back(context->weights, 0);
+             IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, context->actvertex));
+             IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(1) - (2)].edgenum)));
+             IGRAPH_YY_CHECK(igraph_vector_push_back(context->weights, 0));
            ;}
     break;
 
   case 10:
 
     {
-             igraph_vector_push_back(context->vector, context->actvertex);
-             igraph_vector_push_back(context->vector, (yyvsp[(1) - (3)].edgenum));
-             igraph_vector_push_back(context->weights, (yyvsp[(2) - (3)].weightnum));
+             IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, context->actvertex));
+             IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(1) - (3)].edgenum)));
+             IGRAPH_YY_CHECK(igraph_vector_push_back(context->weights, (yyvsp[(2) - (3)].weightnum)));
              context->has_weights = 1;
            ;}
     break;
 
   case 11:
 
-    { igraph_trie_get2(context->trie,
-                                   igraph_lgl_yyget_text(scanner),
-                                   igraph_lgl_yyget_leng(scanner),
-                                   &(yyval.edgenum)); ;}
+    {
+  igraph_integer_t trie_id;
+  IGRAPH_YY_CHECK(igraph_trie_get_len(context->trie,
+    igraph_lgl_yyget_text(scanner),
+    igraph_lgl_yyget_leng(scanner),
+    &trie_id
+  ));
+  (yyval.edgenum) = trie_id;
+;}
     break;
 
   case 12:
 
-    { (yyval.weightnum)=igraph_lgl_get_number(igraph_lgl_yyget_text(scanner),
-                                           igraph_lgl_yyget_leng(scanner)); ;}
+    {
+    igraph_real_t val;
+    IGRAPH_YY_CHECK(igraph_i_parse_real(igraph_lgl_yyget_text(scanner),
+                                        igraph_lgl_yyget_leng(scanner),
+                                        &val));
+    (yyval.weightnum)=val;
+;}
     break;
 
 
@@ -1681,14 +1689,4 @@ int igraph_lgl_yyerror(YYLTYPE* locp, igraph_i_lgl_parsedata_t *context,
   return 0;
 }
 
-igraph_real_t igraph_lgl_get_number(const char *str, long int length) {
-  igraph_real_t num;
-  char *tmp=IGRAPH_CALLOC(length+1, char);
-
-  strncpy(tmp, str, length);
-  tmp[length]='\0';
-  sscanf(tmp, "%lf", &num);
-  IGRAPH_FREE(tmp);
-  return num;
-}
 
