@@ -517,6 +517,16 @@ class GeneratorTests(unittest.TestCase):
         self.assertTrue(df_edges.equals(g_clone.get_edge_dataframe()))
         self.assertTrue(df_vertices.equals(g_clone.get_vertex_dataframe()))
 
+        # pandas Int64 data type
+        edges = pd.DataFrame(np.array([[0, 1], [1, 1], [1, 2]]), dtype="Int64")
+        g = Graph.DataFrame(edges)
+        self.assertTrue(g.vcount() == 3)
+
+        # dataframe with both int data and str data
+        edges = pd.DataFrame({"source": [1, 2, 2], "target": ["A", "B", "A"]})
+        g = Graph.DataFrame(edges, use_vids=False)
+        self.assertTrue(g.vs["name"] == [1, "A", 2, "B"])
+
         # Invalid input
         with self.assertRaisesRegex(ValueError, "two columns"):
             edges = pd.DataFrame({"source": [1, 2, 3]})
@@ -561,10 +571,13 @@ class GeneratorTests(unittest.TestCase):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
             vertices = pd.DataFrame({0: [1, 2, 3]}, index=[0, 1, 2])
             Graph.DataFrame(edges, vertices=vertices)
+        with self.assertRaisesRegex(ValueError, "null"):
+            edges = pd.DataFrame(np.array([[0, 1], [1, np.nan], [1, 2]]), dtype="Int64")
+            Graph.DataFrame(edges)
 
 
 def suite():
-    generator_suite = unittest.makeSuite(GeneratorTests)
+    generator_suite = unittest.defaultTestLoader.loadTestsFromTestCase(GeneratorTests)
     return unittest.TestSuite([generator_suite])
 
 
