@@ -237,28 +237,45 @@ class CentralityTests(unittest.TestCase):
     def testBetweennessCentrality(self):
         g = Graph.Star(5)
         self.assertTrue(g.betweenness() == [6.0, 0.0, 0.0, 0.0, 0.0])
+
         g = Graph(5, [(0, 1), (0, 2), (0, 3), (1, 4)])
         self.assertTrue(g.betweenness() == [5.0, 3.0, 0.0, 0.0, 0.0])
         self.assertTrue(g.betweenness(cutoff=2) == [3.0, 1.0, 0.0, 0.0, 0.0])
         self.assertTrue(g.betweenness(cutoff=1) == [0.0, 0.0, 0.0, 0.0, 0.0])
+
         g = Graph.Lattice([3, 3], circular=False)
         self.assertTrue(
             g.betweenness(cutoff=2) == [0.5, 2.0, 0.5, 2.0, 4.0, 2.0, 0.5, 2.0, 0.5]
         )
 
+        observed = g.betweenness(sources=[0, 8], targets=[0, 8])
+        self.assertEqual(len(observed), g.vcount())
+        for x, y in zip(observed, [0, 1 / 2, 1 / 6, 1 / 2, 2 / 3, 1 / 2, 1 / 6, 1 / 2, 0]):
+            self.assertAlmostEqual(x, y)
+        self.assertRaises(ValueError, g.betweenness, cutoff=2, sources=[0, 8], targets=[0, 8])
+
     def testEdgeBetweennessCentrality(self):
         g = Graph.Star(5)
         self.assertTrue(g.edge_betweenness() == [4.0, 4.0, 4.0, 4.0])
+
         g = Graph(5, [(0, 1), (0, 2), (0, 3), (1, 4)])
         self.assertTrue(g.edge_betweenness() == [6.0, 4.0, 4.0, 4.0])
         self.assertTrue(g.edge_betweenness(cutoff=2) == [4.0, 3.0, 3.0, 2.0])
         self.assertTrue(g.edge_betweenness(cutoff=1) == [1.0, 1.0, 1.0, 1.0])
+
         g = Graph.Ring(5)
         self.assertTrue(g.edge_betweenness() == [3.0, 3.0, 3.0, 3.0, 3.0])
         self.assertTrue(
             g.edge_betweenness(weights=[4, 1, 1, 1, 1]) == [0.5, 3.5, 5.5, 5.5, 3.5]
         )
 
+        g = Graph.Lattice([3, 3], circular=False)
+        observed = g.edge_betweenness(sources=[0, 8], targets=[0, 8])
+        self.assertEqual(len(observed), g.ecount())
+        for x, y in zip(observed, [1 / 2, 1 / 2, 1 / 6, 1 / 3, 1 / 6, 1 / 3, 1 / 6, 1 / 3, 1 / 3, 1 / 2, 1 / 6, 1 / 2]):
+            self.assertAlmostEqual(x, y)
+        self.assertRaises(ValueError, g.edge_betweenness, cutoff=2, sources=[0, 8], targets=[0, 8])
+
     def testClosenessCentrality(self):
         g = Graph.Star(5)
         cl = g.closeness()
@@ -614,18 +631,6 @@ class MiscTests(unittest.TestCase):
         g.to_undirected()
         self.assertRaises(InternalError, g.topological_sorting)
 
-    def testIsDAG(self):
-        g = Graph(5, [(0, 1), (0, 2), (1, 2), (1, 3), (2, 3)], directed=True)
-        self.assertTrue(g.is_dag())
-        g.to_undirected()
-        self.assertFalse(g.is_dag())
-        g = Graph.Barabasi(1000, 2, directed=True)
-        self.assertTrue(g.is_dag())
-        g = Graph.GRG(100, 0.2)
-        self.assertFalse(g.is_dag())
-        g = Graph.Ring(10, directed=True, mutual=False)
-        self.assertFalse(g.is_dag())
-
     def testIsTree(self):
         g = Graph()
         self.assertFalse(g.is_tree())
@@ -853,6 +858,51 @@ class PathTests(unittest.TestCase):
         self.assertEqual(4, sum(1 for path in sps if path[-1] == 12))
         self.assertEqual(12, sum(1 for path in sps if path[-1] == 15))
 
+    def testGetKShortestPaths(self):
+        g = Graph(4, [(0, 1), (1, 2), (1, 3), (2, 4), (3, 4), (4, 5)], directed=True)
+    
+        sps = sorted(g.get_k_shortest_paths(0, 0))
+        expected = [[0]]
+        self.assertEqual(expected, sps)
+    
+        sps = sorted(g.get_k_shortest_paths(0, 5, 2))
+        expected = [[0, 1, 2, 4, 5], [0, 1, 3, 4, 5]]
+        self.assertEqual(expected, sps)
+    
+        sps = sorted(g.get_k_shortest_paths(1, 4, 2))
+        expected = [[1, 2, 4], [1, 3, 4]]
+        self.assertEqual(expected, sps)
+    
+        g = Graph.Lattice([5, 5], circular=False)
+    
+        sps = sorted(g.get_k_shortest_paths(0, 12, 6))
+        expected = [
+            [0, 1, 2, 7, 12],
+            [0, 1, 6, 7, 12],
+            [0, 1, 6, 11, 12],
+            [0, 5, 6, 7, 12],
+            [0, 5, 6, 11, 12],
+            [0, 5, 10, 11, 12],
+        ]
+        self.assertEqual(expected, sps)
+    
+        g = Graph.Lattice([100, 100], circular=False)
+        sps = sorted(g.get_k_shortest_paths(0, 202, 6))
+        expected = [
+            [0, 1, 2, 102, 202],
+            [0, 1, 101, 102, 202],
+            [0, 1, 101, 201, 202],
+            [0, 100, 101, 102, 202],
+            [0, 100, 101, 201, 202],
+            [0, 100, 200, 201, 202],
+        ]
+        self.assertEqual(expected, sps)
+    
+        g = Graph([(0, 1), (1, 2), (0, 2)])
+        g.es["weight"] = [0.5, 0.5, 1]
+        sps = sorted(g.get_k_shortest_paths(0, 2, 2, weights="weight"))
+        self.assertEqual(sorted([[0, 2], [0, 1, 2]]), sorted(sps))
+    
     def testGetAllSimplePaths(self):
         g = Graph.Ring(20)
         sps = sorted(g.get_all_simple_paths(0, 10))
@@ -1046,15 +1096,15 @@ class DominatorTests(unittest.TestCase):
 
 
 def suite():
-    simple_suite = unittest.makeSuite(SimplePropertiesTests)
-    degree_suite = unittest.makeSuite(DegreeTests)
-    local_transitivity_suite = unittest.makeSuite(LocalTransitivityTests)
-    biconnected_suite = unittest.makeSuite(BiconnectedComponentTests)
-    centrality_suite = unittest.makeSuite(CentralityTests)
-    neighborhood_suite = unittest.makeSuite(NeighborhoodTests)
-    path_suite = unittest.makeSuite(PathTests)
-    misc_suite = unittest.makeSuite(MiscTests)
-    dominator_suite = unittest.makeSuite(DominatorTests)
+    simple_suite = unittest.defaultTestLoader.loadTestsFromTestCase(SimplePropertiesTests)
+    degree_suite = unittest.defaultTestLoader.loadTestsFromTestCase(DegreeTests)
+    local_transitivity_suite = unittest.defaultTestLoader.loadTestsFromTestCase(LocalTransitivityTests)
+    biconnected_suite = unittest.defaultTestLoader.loadTestsFromTestCase(BiconnectedComponentTests)
+    centrality_suite = unittest.defaultTestLoader.loadTestsFromTestCase(CentralityTests)
+    neighborhood_suite = unittest.defaultTestLoader.loadTestsFromTestCase(NeighborhoodTests)
+    path_suite = unittest.defaultTestLoader.loadTestsFromTestCase(PathTests)
+    misc_suite = unittest.defaultTestLoader.loadTestsFromTestCase(MiscTests)
+    dominator_suite = unittest.defaultTestLoader.loadTestsFromTestCase(DominatorTests)
     return unittest.TestSuite(
         [
             simple_suite,
